import generate from '@babel/generator';
import { parse } from '@babel/parser';
import traverse from '@babel/traverse';
import * as t from '@babel/types';
import { createFilter } from '@rollup/pluginutils';
import { mkdirSync, readFileSync, writeFileSync } from 'fs';
import * as wxml from 'wxml';

var myTransfer = function (code) {
    var ast = parse(code);
    var pt;
    traverse(ast, {
        enter: function (path) {
            // 存储父级地址
            if (t.isIdentifier(path.node, { name: 'lifetimes' })) {
                pt = path.parentPath;
            }
            // properties重命名
            if (t.isIdentifier(path.node, { name: 'properties' })) {
                path.node.name = 'props';
            }
            // attached重命名并提升层级
            if (t.isIdentifier(path.node, { name: 'attached' })) {
                path.node.name = 'didMount';
                if (pt) {
                    // @ts-ignore
                    pt.insertAfter(path.parentPath.node);
                }
            }
            // detached重命名并提升层级
            if (t.isIdentifier(path.node, { name: 'detached' })) {
                path.node.name = 'didUnmount';
                if (pt) {
                    // @ts-ignore
                    pt.insertAfter(path.parentPath.node);
                }
            }
        },
        exit: function (path) {
            // @ts-ignore
            if (t.isIdentifier(path.node.key, { name: 'lifetimes' })) {
                path.remove();
            }
        }
    });
    var output = generate(ast, {}, code);
    return output;
};

function jsTransfer(_a) {
    var include = _a.include, exclude = _a.exclude, platform = _a.platform;
    var filter = createFilter(include, exclude);
    return {
        name: 'minip-components-jsTransfer',
        transform: function (code, id) {
            if (filter(id)) {
                if (platform === 'my') {
                    return myTransfer(code);
                }
                else {
                    return { code: code };
                }
            }
            else {
                return null;
            }
        }
    };
}

var MYENUMS = {
    'wx:if': 'a:if',
    'wx:elif': 'a:elif',
    'wx:else': 'a:else',
    'wx:for': 'a:for',
    'wx:for-index': 'a:for-index',
    'wx:for-item': 'a:for-item',
    'wx:key': 'a:key',
    bindtap: 'onTap',
    binderror: 'onError',
    bindload: 'onLoad',
    bindchange: 'onChange',
    catchtouchmove: 'catchTouchMove'
};

var QQENUMS = {
    'wx:if': 'wx:if',
    'wx:elif': 'wx:elif',
    'wx:else': 'wx:else',
    'wx:for': 'wx:for',
    'wx:for-index': 'wx:for-index',
    'wx:for-item': 'wx:for-item',
    'wx:key': 'wx:key',
    bindtap: 'bindtap',
    binderror: 'binderror',
    bindload: 'bindload',
    bindchange: 'bindchange',
    catchtouchmove: 'catchtouchmove'
};

var SWANENUMS = {
    'wx:if': 's-if',
    'wx:elif': 's-elif',
    'wx:else': 's-else',
    'wx:for': 's-for',
    'wx:for-index': 's-for-index',
    'wx:for-item': 's-for-item',
    'wx:key': 's-key',
    bindtap: 'bindtap',
    binderror: 'binderror',
    bindload: 'bindload',
    bindchange: 'bindchange',
    catchtouchmove: 'catchtouchmove'
};

var TTENUMS = {
    'wx:if': 'tt:if',
    'wx:elif': 'tt:elif',
    'wx:else': 'tt:else',
    'wx:for': 'tt:for',
    'wx:for-index': 'tt:for-index',
    'wx:for-item': 'tt:for-item',
    'wx:key': 'tt:key',
    bindtap: 'bindtap',
    binderror: 'binderror',
    bindload: 'bindload',
    bindchange: 'bindchange',
    catchtouchmove: 'catchtouchmove'
};

var M = { my: MYENUMS, qq: QQENUMS, swan: SWANENUMS, tt: TTENUMS };
var platformTransfer = function (ast, platform) {
    var ENUMS = M[platform];
    var nodesErgodic = function (ast) {
        return ast.map(function (n) {
            var ns = n;
            if (n.attributes) {
                Object.keys(n.attributes).forEach(function (k) {
                    // 修改属性名
                    var nk = ENUMS[k];
                    if (nk) {
                        ns.attributes[nk] = n.attributes[k];
                        delete ns.attributes[k];
                    }
                });
            }
            if (n.childNodes) {
                ns.childNodes = nodesErgodic(n.childNodes);
            }
            return ns;
        });
    };
    return nodesErgodic(ast);
};
var bundleSuffix = {
    wx: '.wxml',
    my: '.axml',
    swan: '.swan',
    tt: '.ttml',
    qq: '.qml'
};

function wxmlTransfer(_a) {
    var platform = _a.platform, template = _a.template, output = _a.output, name = _a.name;
    return {
        name: 'minip-components-wxmlTransfer',
        writeBundle: function () {
            var code = readFileSync(template).toString();
            var bundle = wxml.parse(code);
            if (platform !== 'wx') {
                bundle = platformTransfer(bundle, platform);
            }
            if (!output.endsWith('/')) {
                output = "".concat(output, "/");
            }
            mkdirSync("".concat(output).concat(name, "/"), { recursive: true });
            writeFileSync("".concat(output).concat(name, "/").concat(name).concat(bundleSuffix[platform]), wxml.serialize(bundle[0]));
        }
    };
}

var index = { jsTransfer: jsTransfer, wxmlTransfer: wxmlTransfer };

export { index as default };
